# rubocop:disable AbcSize
require 'plate_recognizer/version'
require 'net/http'

module PlateRecognizer
  class Error < StandardError; end

  class Output
    attr_accessor :processing_time, :version, :filename, :camera_id
    attr_accessor :timestamp
    attr_accessor :results

    def initialize(opts)
      @processing_time = opts['processing_time']
      @filename = opts['filename']
      @version = opts['version']
      @camera_id = opts['camera_id']
      @timestamp = opts['timestamp']

      @results = []
      opts['results'].each do |result_hash|
        @results << Result.new(result_hash)
      end
    end
  end

  class Result
    attr_accessor :box
    attr_accessor :plate, :region, :vehicle, :score, :candidates, :ds_score

    def initialize(opts)
      @plate = opts['plate']
      @score = opts['score']
      @dscore = opts['dscore']

      @box = Box.new(opts.fetch('box'))
      @region = Region.new(opts['region'])
      @vehicle = Vehicle.new(opts['vehicle'])

      @candidates = []
      opts['candidates'].each do |candidate_hash|
        @candidates << Candidate.new(candidate_hash)
      end
    end
  end

  class Box
    attr_accessor :ymin, :xmin, :ymax, :xmax

    def initialize(opts)
      @ymin = opts.fetch('ymin')
      @xmin = opts.fetch('xmin')
      @ymax = opts.fetch('ymax')
      @xmax = opts.fetch('xmax')
    end

    def width
      xmax - xmin
    end

    def height
      ymax - ymin
    end

    def polygon
      [
        "#{xmin},#{ymin}",
        "#{xmin},#{ymax}",
        "#{xmax},#{ymax}",
        "#{xmax},#{ymin}"
      ].join(' ')
    end
  end

  class Candidate
    attr_accessor :score, :plate

    def initialize(opts)
      return unless opts

      @score = opts['score']
      @plate = opts['plate']
    end
  end

  class Vehicle
    attr_accessor :score, :type

    def initialize(opts)
      return unless opts

      @score = opts['score']
      @type = opts['type']
    end
  end

  class Region
    attr_accessor :score, :code

    def initialize(opts)
      return unless opts

      @score = opts['score']
      @type = opts['code']
    end
  end

  mattr_accessor :api_key
  @@api_key = nil

  def self.setup
    yield self
  end

  def self.process(image_path)
    raise 'Plate Recognizer API must be set!' if PlateRecognizer.api_key.blank?

    url = URI.parse('https://api.platerecognizer.com/v1/plate-reader')
    req = Net::HTTP::Post.new(url.path)
    req['Accept'] = 'application/json'
    req['Authorization'] = "TOKEN #{PlateRecognizer.api_key}"
    form_data = [['upload', File.open(image_path)]]
    req.set_form form_data, 'multipart/form-data'

    sock = Net::HTTP.new(url.host, url.port)
    sock.use_ssl = true
    response = sock.start { |http| http.request(req) }

    begin
      Output.new(JSON.parse(response.body))
    rescue
      nil
    end
  end
end
# rubocop:enable all
